/*
# $BEGIN_SYNCGENIE_LICENSE$
# 
# This file is part of the SyncGenie project, a library for the Arduino IDE
# for interfacing with the the Teensy microcontroller via a SyncGenie
# breakout board (see https://osf.io/r9pb6/ ), with an emphasis on time
# synchronization in neuroscience experiments.
# 
# Author: Jeremy Hill (2024-)
# 
# No Copyright
# ============
# The author has dedicated this work to the public domain under the terms of
# Creative Commons' CC0 1.0 Universal legal code, waiving all of his rights to
# the work worldwide under copyright law, including all related and neighboring
# rights, to the extent allowed by law.
# 
# You can copy, modify, distribute and perform the work, even for commercial
# purposes, all without asking permission. See Other Information below.
# 
# Other Information
# =================
# In no way are the patent or trademark rights of any person affected by CC0,
# nor are the rights that other persons may have in the work or in how the work
# is used, such as publicity or privacy rights.
# 
# The author makes no warranties about the work, and disclaims liability for
# all uses of the work, to the fullest extent permitted by applicable law. When
# using or citing the work, you are requested to preserve the author attribution
# and this copyright waiver, but you should not imply endorsement by the author.
# 
# $END_SYNCGENIE_LICENSE$
*/

/*
This sketch, and the accompanying header, illustrate the virtual class `Trigger`
can be subclassed. This example subclass triggers based changes detected by an
MPU6050 accelerometer/gyroscope unit, connected to the SyncGenie via I2C. The
SyncGenie's JST pins (from left to right) need to be connected to the MPU6050's
GND, VCC, SDA and SCL in that order.
*/

#include <SyncGenie.h>
SyncGenie genie;

#include "TriggerMPU6050.h" // this header, included in this example sketch directory, illustrates how to make a virtual `Trigger` subclass
TriggerMPU6050 accel( genie.D0_OUT );

void setup()
{
	genie.begin( "Demo sketch:\nTrigger MPU6050" ); // this initializes some pin modes, starts the LCD screen, starts the Serial connection (if available), and flickers D9_LED (same as LED_BUILTIN) to say hello

	accel.begin(); // unlike the `Trigger` base class, this `begin()` call has to be made explicitly here because it does things that are not safe to do in the global constructor
	
	// Change these parameters as you see fit:
	accel.threshold = 2000;
	accel.minimumPulseDurationInMicros   = 250000; // use an output pulse duration of 250ms
	accel.pulseDurationIncrementInMicros = 250000; // extend the pulse 250ms at a time if the signal is still coming
	accel.refractoryPeriodInMillis       = 500;    // debounce
}

void loop()
{
	static Trigger & trigger( accel );  // it's a virtual class, so you can refer to it via a reference or pointer to its base type and it will still work correctly

	unsigned long now = micros();		
	trigger.process( now ); // almost the same as `trigger.read(now) && trigger.write(now)` (the only difference: the return value of `process()` always reflects the pulse output state, even if `read()` returned `false`)
}
