/*
# $BEGIN_SYNCGENIE_LICENSE$
# 
# This file is part of the SyncGenie project, a library for the Arduino IDE
# for interfacing with the the Teensy microcontroller via a SyncGenie
# breakout board (see https://osf.io/r9pb6/ ), with an emphasis on time
# synchronization in neuroscience experiments.
# 
# Author: Jeremy Hill (2024-)
# 
# No Copyright
# ============
# The author has dedicated this work to the public domain under the terms of
# Creative Commons' CC0 1.0 Universal legal code, waiving all of his rights to
# the work worldwide under copyright law, including all related and neighboring
# rights, to the extent allowed by law.
# 
# You can copy, modify, distribute and perform the work, even for commercial
# purposes, all without asking permission. See Other Information below.
# 
# Other Information
# =================
# In no way are the patent or trademark rights of any person affected by CC0,
# nor are the rights that other persons may have in the work or in how the work
# is used, such as publicity or privacy rights.
# 
# The author makes no warranties about the work, and disclaims liability for
# all uses of the work, to the fullest extent permitted by applicable law. When
# using or citing the work, you are requested to preserve the author attribution
# and this copyright waiver, but you should not imply endorsement by the author.
# 
# $END_SYNCGENIE_LICENSE$
*/

#ifndef   __SerialTriggerOutput_H__
#define   __SerialTriggerOutput_H__

#include <stdint.h>
#include <Arduino.h>

class SerialTriggerOutput
{
	public:
		// An object for managing a trigger for systems that expect a UART serial message. If `baud` is non-zero, it will be passed to the `HardwareSerial` instance's `begin()` method during object construction (pass 0 if you want to call `.port.begin( ... )` manually later).
		SerialTriggerOutput( HardwareSerial & serialPort=Serial1, int baud=0 );
		~SerialTriggerOutput();
	
		// Initializes the serial port. (If you supplied a non-zero baud rate to the object constructor, this will have been done automatically already and does not need to be called manually.)
		void begin( int baud=115200 );
		
		// Calls end() on the serial port member instance to stop serial communication.
		void end( void );
	
		// Assign a value (by default, 0) to the 8-bit trigger value.
		void reset( uint8_t value=0 );
		
		// Manipulate one specified bit of the trigger byte (`bitPosition` from 0 to 7).
		bool setBit( unsigned int bitPosition, bool value=true );
		
		// If the composite trigger byte has changed since it was last sent, and is now non-zero, send it.  If it has changed but is now zero, only send if `evenIfZero` is `true`. Return `true` if a byte was sent.
		bool send( bool evenIfZero=false );
	
		HardwareSerial & port; // UART serial object
		String debug; // For best performance, leave this empty. If it is non-empty, event messages will be echoed as text to the main (USB) serial port.
	private:
		uint8_t mComposed;
		uint8_t mSent;
};

#endif // __SerialTriggerOutput_H__
