/*
# $BEGIN_SYNCGENIE_LICENSE$
# 
# This file is part of the SyncGenie project, a library for the Arduino IDE
# for interfacing with the the Teensy microcontroller via a SyncGenie
# breakout board (see https://osf.io/r9pb6/ ), with an emphasis on time
# synchronization in neuroscience experiments.
# 
# Author: Jeremy Hill (2024-)
# 
# No Copyright
# ============
# The author has dedicated this work to the public domain under the terms of
# Creative Commons' CC0 1.0 Universal legal code, waiving all of his rights to
# the work worldwide under copyright law, including all related and neighboring
# rights, to the extent allowed by law.
# 
# You can copy, modify, distribute and perform the work, even for commercial
# purposes, all without asking permission. See Other Information below.
# 
# Other Information
# =================
# In no way are the patent or trademark rights of any person affected by CC0,
# nor are the rights that other persons may have in the work or in how the work
# is used, such as publicity or privacy rights.
# 
# The author makes no warranties about the work, and disclaims liability for
# all uses of the work, to the fullest extent permitted by applicable law. When
# using or citing the work, you are requested to preserve the author attribution
# and this copyright waiver, but you should not imply endorsement by the author.
# 
# $END_SYNCGENIE_LICENSE$
*/

#ifndef   __Switch_CPP__
#define   __Switch_CPP__

#include <limits.h> // for LONG_MIN and LONG_MAX

#include "Switch.h"

Switch::Switch( int pin, int closedValue, int referencePin, bool pull )
: threshold( 0 ), debounceMicros( 25000 ), lastEvent( 0 ),
  mPin( pin ), mBaselineValue( closedValue == HIGH ? LOW : HIGH ), mReferencePin( referencePin ), mPull( pull ),
  mWasClosed( false ), mReleaseIsPending( false ), mReleaseTimeInMicros( 0 )
{
	this->begin();
}

Switch::~Switch()
{
	
}

void
Switch::begin( void )
{
	if( mPin >= 0 )
		pinMode( mPin, ( !mPull ) ? INPUT : ( mBaselineValue == HIGH ) ? INPUT_PULLUP : INPUT_PULLDOWN );
	if( mReferencePin >= 0 )
	{
		pinMode( mReferencePin, OUTPUT );
		digitalWrite( mReferencePin, ( mBaselineValue == HIGH ) ? LOW : HIGH );
	}
	mWasClosed = this->isClosed();
	mReleaseIsPending = false;
}

bool
Switch::isClosed( void ) const
{
	if( mPin < 0 ) return false;
	if( this->threshold ) return (
		mBaselineValue == LOW
		? ::analogRead( mPin ) >= this->threshold
		: ::analogRead( mPin ) <  this->threshold
	);
	return ::digitalRead( mPin ) != mBaselineValue;
}

int
Switch::event( void )
{
	int result;
	bool isClosedNow = this->isClosed();
	if( this->debounceMicros ) // debounce release-and-press
	{
		if( isClosedNow ) mReleaseIsPending = false;
		else if( mWasClosed )
		{
			unsigned long now = micros();
			if( mReleaseIsPending )
			{
				isClosedNow = ( ( now - mReleaseTimeInMicros ) < this->debounceMicros );
				if( !isClosedNow ) mReleaseIsPending = false;
			}
			else
			{
				mReleaseIsPending = true;
				mReleaseTimeInMicros = now;
				isClosedNow = true;
			}
		}
	}
	if(       isClosedNow && !mWasClosed ) result = this->lastEvent = +1; // button press
	else if( !isClosedNow &&  mWasClosed ) result = this->lastEvent = -1; // button release
	else result = 0; // no change
	// TODO: implement long-press detection? (+2 and -2)
	mWasClosed = isClosedNow;
	return result;
}

int
Switch::analogRead( void ) const
{
	if( mPin < 0 ) return 0;
	return ::analogRead( mPin );
}

Wheel::Wheel( int pinA, int pinB )
: lastEvent( 0 ), counter( 0 ), minCounter( LONG_MIN ), maxCounter( LONG_MAX ),
  mPinA( pinA ), mPinB( pinB ), mPreviousState( 0 )
{
	this->begin();
}

Wheel::~Wheel() { }

void
Wheel::begin( void )
{
	if( mPinA >= 0 ) { pinMode( mPinA, INPUT_PULLUP ); mPreviousState = digitalRead( mPinA ); }
	if( mPinB >= 0 ) { pinMode( mPinB, INPUT_PULLUP ); }
}

void
Wheel::begin( long min, long max )
{
	this->begin();
	this->setRange( min, max );
}
	
void
Wheel::setRange( long min, long max )
{
	this->minCounter = min;
	this->maxCounter = max;
	if( this->counter < this->minCounter ) this->counter = this->minCounter;
	if( this->counter > this->maxCounter ) this->counter = this->maxCounter;
}
int
Wheel::event( void )
{
	if( mPinA < 0 || mPinB < 0 ) return 0;
	int valueA = digitalRead( mPinA );
	if( valueA == mPreviousState ) return 0;
	mPreviousState = valueA;
	if( digitalRead( mPinB ) == valueA ) { if( this->counter-- <= this->minCounter ) { this->counter = this->minCounter; } return ( this->lastEvent = -1 ); }
	else                                 { if( this->counter++ >= this->maxCounter ) { this->counter = this->maxCounter; } return ( this->lastEvent = +1 ); }
}


Gauge::Gauge( int pin, unsigned int thresholdValue )
: threshold ( thresholdValue ), lastReading( 0 ), mPin( pin ), mLastTimestamp( 0 ),
  mTotal( 0 ), mNumberOfSamples( 0 )
{
	this->begin();
}

Gauge::~Gauge()
{
	
}
		
void
Gauge::begin( void )
{
	if( mPin >= 0 )
	{
		pinMode( mPin, INPUT );
		this->lastReading = ::analogRead( mPin );
	}
	mLastTimestamp = ::micros();
	mTotal = this->lastReading;
	mNumberOfSamples = 0;
}
		
int
Gauge::changed( unsigned int microsecondTimestamp )
{
	int sample = ::analogRead( mPin );
	
	mTotal += sample; mNumberOfSamples++;            // also works reasonably well if you set `currentReading = sample` but then you need 
	int currentReading = mTotal / mNumberOfSamples;  // a higher threshold (5000 might be good) and small oscillatory changes will happen
	
	int deltaV = currentReading - this->lastReading;
	unsigned int deltaT = microsecondTimestamp - mLastTimestamp;
	unsigned int absV = ( unsigned int )( 1 + abs( deltaV ) );
	if( ( absV * deltaT ) > this->threshold * 1000 * 2 )
	{
		this->lastReading = currentReading;
		mLastTimestamp = microsecondTimestamp;
		mTotal = 0; mNumberOfSamples = 0;
		if( deltaV && this->debug.length() ) { Serial.print( this->debug ); Serial.print( ": " ); Serial.println( currentReading ); Serial.flush(); }
		return deltaV;
	}
	return 0;
}

#endif // __Switch_CPP__

