/*
# $BEGIN_SYNCGENIE_LICENSE$
# 
# This file is part of the SyncGenie project, a library for the Arduino IDE
# for interfacing with the the Teensy microcontroller via a SyncGenie
# breakout board (see https://osf.io/r9pb6/ ), with an emphasis on time
# synchronization in neuroscience experiments.
# 
# Author: Jeremy Hill (2024-)
# 
# No Copyright
# ============
# The author has dedicated this work to the public domain under the terms of
# Creative Commons' CC0 1.0 Universal legal code, waiving all of his rights to
# the work worldwide under copyright law, including all related and neighboring
# rights, to the extent allowed by law.
# 
# You can copy, modify, distribute and perform the work, even for commercial
# purposes, all without asking permission. See Other Information below.
# 
# Other Information
# =================
# In no way are the patent or trademark rights of any person affected by CC0,
# nor are the rights that other persons may have in the work or in how the work
# is used, such as publicity or privacy rights.
# 
# The author makes no warranties about the work, and disclaims liability for
# all uses of the work, to the fullest extent permitted by applicable law. When
# using or citing the work, you are requested to preserve the author attribution
# and this copyright waiver, but you should not imply endorsement by the author.
# 
# $END_SYNCGENIE_LICENSE$
*/

#ifndef   __Switch_H__
#define   __Switch_H__

#include <Arduino.h> // for pinMode(), digitalRead(), analogRead(), HIGH, LOW, INPUT, OUTPUT, etc.

class Switch
{
	public:
		// A class that represents a simple switch. The input pin will be put into `INPUT`, `INPUT_PULLUP` or `INPUT_PULLDOWN` mode according to the `closedValue` and `pull` settings. Default settings get you a single-pin pullup that is assumed to get shorted to ground when it is closed. If a `referencePin` is specified, it will be put into `OUTPUT` mode and set with `digitalWrite(referencePin, closedValue)` - then it is assumed that closing the switch will short the input pin to the reference pin.
		Switch( int pin, int closedValue=LOW, int referencePin=-1, bool pull=true );
		~Switch();
	
		// Sets pin modes. Usually does not need to be called explicitly since it is called during construction.
		void begin( void );
		
		// Reads from the switch pin and returns whether or not the switch is currently closed (pressed). 
		bool isClosed( void ) const;
		
		// Reads from the switch pin and returns whether or not the switch is currently closed (pressed). 
		inline bool isPressed( void ) const { return isClosed(); } // it's an alias.  `isClosed()` semantics make more sense from the circuit perspective, `isPressed()` semantics make more sense from the user-interface perspective.
		
		// Returns +1 if the switch has been closed (pressed), -1 if it has been opened (released), or 0 if there has been no change, since the previous call to `event()`.
		int  event( void );
	
		// Reads an analog value from the switch.
		int  analogRead( void ) const;
	
		int threshold; // The default value of 0 is suitable for digital switches. Any other value causes `analogRead()` to be used instead of `digitalRead()` (for example, for a phototransistor).
		unsigned long debounceMicros; // The switch has to be released for at least this many microseconds to cause `event()` to return -1.
		int lastEvent;
	
		static const int PRESS   = +1; // possible return value of `event()`
		static const int RELEASE = -1; // possible return value of `event()`
		// TODO: implement long-press detection? (+2 and -2)
	
	private:
		int           mPin;
		int           mBaselineValue;
		int           mReferencePin;
		bool          mPull;
		bool          mWasClosed;
		bool          mReleaseIsPending;
		unsigned long mReleaseTimeInMicros;
};

class Wheel
{
	public:
		// A class that represents the wheel of a rotary encoder.
		Wheel( int pinA, int pinB );
		~Wheel();
	
		// Sets pin modes. Usually does not need to be called explicitly since it is called during construction.
		void begin( void );
		// Sets pin modes (just like `begin()` without args) and also `setRange()`.
		void begin( long min, long max );
	
		// Sets and asserts the minimum and maximum values that the `counter` member variable can take.
		void setRange( long min, long max );
	
		// Returns +1 if the wheel has been moved clockwise, -1 if it has been moved counter-clockwise, or 0 if there has been no change, since the previous call to `event()`. Updates `counter` as appropriate.
		int event( void );
	
		int lastEvent;
		long counter;    // This is a running total of the number of clockwise clicks minus the number of counter-clockwise clicks.
		long minCounter; // The counter will not be decremented beyond this number (defaults to `LONG_MIN`).
		long maxCounter; // The counter will not be incremented beyond this number (defaults to `LONG_MAX`).
	
		static const int LEFT  = -1; // possible return value of `event()`
		static const int RIGHT = +1; // possible return value of `event()`
	
	private:
		int mPinA;
		int mPinB;
		int mPreviousState;
};

class Gauge
{
	public:
		// An object that represents a typically-flat but nonetheless variable signal like a potentiometer reading. The `pin` should be an analog input pin. The `thresholdValue` is in ADC units times milliseconds, and is roughly the number of milliseconds a change of magnitude 1 can be ignored (a change of magnitude 10 will only be ignored for 1/10 of that time). 
		Gauge( int pin, unsigned int thresholdValue=1000 );
		~Gauge();
		operator int() const { return this->lastReading; }
		
		// Sets pin mode. Usually does not need to be called explicitly since it is called during construction.
		void begin( void );
		
		// Returns any change in the reading (since the since the last change) that is large and/or sustained enough to cross the threshold. Note that the return value is a delta (the absolute value of the new reading can be obtained from the `.lastReading` member, or just by casting the `Gauge` instance as an `int`).
		int changed( unsigned int microsecondTimestamp );
		
		unsigned int threshold; // measured in milliseconds * ADC units
		int lastReading;
		String debug; // For best performance, leave this empty. If it is non-empty, event messages will be echoed as text to the main (USB) serial port.
		
	private:
		int mPin;
		unsigned int mLastTimestamp;
		int mTotal;
		int mNumberOfSamples;
};
#endif // __Switch_H__
