/*
# $BEGIN_SYNCGENIE_LICENSE$
# 
# This file is part of the SyncGenie project, a library for the Arduino IDE
# for interfacing with the the Teensy microcontroller via a SyncGenie
# breakout board (see https://osf.io/r9pb6/ ), with an emphasis on time
# synchronization in neuroscience experiments.
# 
# Author: Jeremy Hill (2024-)
# 
# No Copyright
# ============
# The author has dedicated this work to the public domain under the terms of
# Creative Commons' CC0 1.0 Universal legal code, waiving all of his rights to
# the work worldwide under copyright law, including all related and neighboring
# rights, to the extent allowed by law.
# 
# You can copy, modify, distribute and perform the work, even for commercial
# purposes, all without asking permission. See Other Information below.
# 
# Other Information
# =================
# In no way are the patent or trademark rights of any person affected by CC0,
# nor are the rights that other persons may have in the work or in how the work
# is used, such as publicity or privacy rights.
# 
# The author makes no warranties about the work, and disclaims liability for
# all uses of the work, to the fullest extent permitted by applicable law. When
# using or citing the work, you are requested to preserve the author attribution
# and this copyright waiver, but you should not imply endorsement by the author.
# 
# $END_SYNCGENIE_LICENSE$
*/

#ifndef   __SyncGenie_H__
#define   __SyncGenie_H__

#ifndef SYNCGENIE_BOARD_REVISION
#	include "SyncGenieBoardRevision.h"
#endif
#define QUOTE_( X )          #X
#define QUOTE( X )           QUOTE_( X )
#define CONCAT3_( X, Y, Z )  X ## Y ## Z
#define CONCAT3( X, Y, Z )   CONCAT3_( X, Y, Z )
#define PIN_HEADER_FILE      CONCAT3( SyncGenie, SYNCGENIE_BOARD_REVISION, Pins.h )

#include <Arduino.h>
#include <LiquidCrystal.h>

#include <Switch.h> // Switch and Wheel classes - TODO: maybe move this, along with Triggering.h and friends, to an entirely separate Triggering library and make it a dependency of this one

#define NO_PIN        (-1)
#define NO_INPUT_PIN  (-1)
#define NO_OUTPUT_PIN (-1)

class SyncGenie
{
	public:
#		include QUOTE( PIN_HEADER_FILE )
	
		SyncGenie() : // need this here in the header, to allow SYNCGENIE_BOARD_REVISION shenanigans to work
			lcd( LCD_RS, LCD_EN, LCD_D4, LCD_D5, LCD_D6, LCD_D7 ),
			pushButton( BUTTON ),
			selectorButton( SELECTOR_BUTTON ),
			selectorWheel( SELECTOR_A, SELECTOR_B )
		{} 
		~SyncGenie();
		
		// `genie.begin()` or `genie.begin( "LCD msg" )` should be called in your `setup()` function
		void begin( void );
		// `genie.begin()` or `genie.begin( "LCD msg" )` should be called in your `setup()` function
		void begin( const String & msg );
		
		// Uses the 10-LED bar-graph as a bar-graph, and illuminates the first `height` bars (NB: the first 8 LEDs are hard-wired to D-SUB outputs, so voltages will be measurable there too).
		int bar( int height );
		
		// Flickers an LED. Called at the end of `genie.begin()`.
		static void flicker( int ledPin=LED_BUILTIN, unsigned long millisOn=50, unsigned long millisOff=50, unsigned long millisTotal=1000 );
		
		// Read the temperature sensor (Teensy 4.x only)
		float temperature( void );
		
		// Displays the specified message on the LCD if there is one. This is almost the same as `genie.lcd.print()` except (a) you can display a multi-line string by including `\n`, and (b) certain strings are interpolated (specifically: `@TEENSY_VERSION@`, `@TEENSY_LENGTH@` and `@BOARD_REVISION@`).
		void display( const String & msg, int row=0, bool echoToSerial=false );
		
		// Point the SyncGenie to a static string that can then be displayed to the user on the LCD screen using `showMessage()` and `scrollMessage()`.  The message will be word-wrapped automatically. Include the `\n` character in your message to force a line break (only one, though). The `\t` character will not be rendered at all, but it marks a discretionary place where a word could be broken (it's good to put one after a hyphen).
		void loadMessage( const char * msg );
		
		// After `loadMessage()`, display as much of the message as will currently fit on the LCD screen, word-wrapped.
		void showMessage( void );
		
		// After `loadMessage()` and `showMessage()`, scroll the message by the specified number of lines (positive for forwards, negative for backwards) and re-display. This will not work well if the message content changes.
		void scrollMessage( int lines );

		LiquidCrystal lcd;
		Switch pushButton;
		Switch selectorButton;
		Wheel  selectorWheel;
		
	private:
		const char * mMessageStart;
		const char * mMessageCursor;
		const char * messageLine( const char * start, int lcdRow=-1 );
		
};

#endif // __SyncGenie_H__
